require 'thor'
require 'quandl/slack'

Quandl::Slack.generate_notifier('ci')

module Quandl
  module Utility
    module ThorTasks
      # Tasks to run on CI Server
      class CI < Thor
        namespace 'qutil:ci'

        desc 'notify_slack', 'Notify Quandl of build status of CI'
        def notify_slack(message = nil)
          notification = message.nil? ? ci_build_message : "#{ci_build_message} - #{message}"
          Quandl::Slack::Ci.ping(notification, channel: slack_channel, icon_emoji: slack_emoji)
        end

        private

        # Generate a message for the CI build that will be sent to Quandl
        #
        # Returns string
        def ci_build_message
          "*#{current_project}* (#{git_branch}) *#{tddium_build_status}* - #{tddium_report_url} - (#{git_user})"
        end

        # Which slack channel to send build status to.
        # Defaults to 'development', but it can be overriden in tddium.yml
        #
        # Returns string
        def slack_channel
          ENV.fetch('SLACK_CHANNEL', '#development')
        end

        # The URL for the build that just ran
        #
        # Returns string
        def tddium_report_url
          "https://#{tddium_api_server}/1/reports/#{tddium_session_id}"
        end

        def tddium_build_status
          ENV['TDDIUM_BUILD_STATUS']
        end

        def tddium_api_server
          ENV['TDDIUM_API_SERVER']
        end

        def tddium_session_id
          ENV['TDDIUM_SESSION_ID']
        end

        def current_project
          ENV['QUANDL_PROJECT']
        end

        def git_branch
          @git_branch ||= `git branch`.chomp.gsub(/\*/, '').strip
        end

        def git_user
          @git_user ||= `git --no-pager show -s --format='%an'`.chomp
        end

        # Did the build end with failed status?
        def failed?
          ENV['TDDIUM_BUILD_STATUS'] == 'failed'
        end

        # Did the build end with success status?
        def passed?
          ENV['TDDIUM_BUILD_STATUS'] == 'passed'
        end

        def error?
          !(failed? || passed?)
        end

        # Use different emojis for build statuses.
        #
        # Returns string
        def slack_emoji
          if failed?
            fail_emoji
          elsif passed?
            pass_emoji
          else
            error_emoji
          end
        end

        # Return the emoji to use for an error build
        # Can be overriden with environment variables in tddium.yml.
        # Default is :warning:
        #
        # Returns string
        def error_emoji
          ENV.fetch('SLACK_ERROR_EMOJI', ':warning:')
        end

        # Return the emoji to use for a failed build
        # Can be overriden with environment variables in tddium.yml.
        # Default is :scream_cat:
        #
        # Returns string
        def fail_emoji
          ENV.fetch('SLACK_FAIL_EMOJI', ':scream_cat:')
        end

        # Return the emoji to use for a successful build
        # Can be overriden with environment variables in tddium.yml.
        # Default is :thumbsup:
        #
        # Returns string
        def pass_emoji
          ENV.fetch('SLACK_PASS_EMOJI', ':thumbsup:')
        end
      end
    end
  end
end
